@@ -22,12 +22,14 @@ import math
 import logging
 import numpy as np
 
-from biosignalml.ontology import BSML
+from biosignalml.model import BSML
 from biosignalml.model.core import AbstractObject
 from biosignalml.model.mapping import PropertyMap
 
 from biosignalml.rdf import XSD
 
+__all__ = [ 'DataError', 'Clock', 'DataSegment', 'TimeSeries', 'UniformClock', 'UniformTimeSeries' ]
+
 
 class DataError(Exception):
 #==========================
@@ -43,7 +45,7 @@ class Clock(AbstractObject):
   :param float resolution: The time, in seconds, represented by a time of 1.0. Optional.
   :param float rate: Resolution can be given as the reciprocal of ``rate``. Optional.
   """
-  metaclass = BSML.SampleClock
+  metaclass = BSML.SampleClock  #: :attr:`.BSML.SampleClock`
 
   # Also have 'frequency' ?? and/or 'period' ??
   attributes = [ 'resolution', 'rate' ]
