@@ -22,15 +22,16 @@ RelativeTimeLine, Instant and Interval objects.
 ######################################################
 
 
-from biosignalml import BSML
-
 import biosignalml.rdf as rdf
 from biosignalml.rdf import RDF, RDFS, DCT, XSD, TL
 
 import biosignalml.utils as utils
 import biosignalml.model as model
+from biosignalml.model import BSML
 from biosignalml.model.mapping import PropertyMap
 
+__all__ = [ 'Instant', 'Interval', 'RelativeTimeLine', 'TemporalEntity' ]
+
 
 class RelativeTimeLine(model.core.AbstractObject):
 #=================================================
@@ -65,6 +66,14 @@ class TemporalEntity(model.core.AbstractObject):
                                       to_rdf=PropertyMap.get_uri,
                                       from_rdf=RelativeTimeLine) }
 
+  @classmethod
+  def create(cls, uri, start, duration=None, end=None, **kwds):
+  #------------------------------------------------------------
+    if duration in [None, 0.0] and end in [None, start]:
+      return Instant(uri, start, **kwds)
+    else:
+      return Interval(uri, start, duration=duration, end=end, **kwds)
+
   @classmethod
   def create_from_graph(cls, uri, graph, **kwds):
   #----------------------------------------------
@@ -114,7 +123,7 @@ class Interval(TemporalEntity):
   def end(self, timeline=None):     # Needs to use timeline to map
   #----------------------------
     '''Get the end of the interval.'''
-    return self.start + (self.duration if self.duration is not None else None)
+    return self.start + (self.duration if self.duration is not None else 0.0)
 
   def __add__(self, increment):
   #----------------------------
