@@ -18,14 +18,24 @@
 #
 ######################################################
 
+"""
+BSML Store description...
+
+"""
+
 import logging
+import operator
 
 import biosignalml.formats
-from biosignalml import BSML, Recording, Signal, Event, Annotation
-from biosignalml.rdf import RDFS, DCT, PRV, Format
+from biosignalml import BSML, Recording, Signal, Event, Annotation, Segment
+from biosignalml.data.time import TemporalEntity
+from biosignalml.rdf import RDFS, DCT, PRV, TL, Format
 import biosignalml.rdf.sparqlstore as sparqlstore
 
-from graphstore import GraphStore, GraphUpdate
+from .graphstore import GraphStore, GraphUpdate
+
+__all__ = [ 'BSMLStore', 'BSMLUpdateStore' ]
+
 
 '''Graph holding bsml:SemanticTag resources.'''
 SEMANTIC_TAGS = 'http://ontologies.biosignalml.org/semantic-tags'
@@ -103,6 +113,7 @@ class BSMLStore(GraphStore):
                             str(self.get_objects(rec_uri, DCT.format, graph=graph_uri)[0]),
                             Recording)
       graph = self.get_resource_as_graph(rec_uri, BSML.Recording, graph_uri)
+##    graph = self.get_graph(graph_uri, BSML.Recording)
       rec = recording_class.create_from_graph(rec_uri, graph, signals=False, **kwds)
       if rec is not None:
         if with_signals:
@@ -124,12 +135,15 @@ class BSMLStore(GraphStore):
     :param graph_uri: An optional URI of the graph to query.
     :rtype: :class:`~biosignalml.Signal`
     '''
-    ### This assumes all signals for arecording in a repository
+    ### This assumes all signals for a recording in the repository
     ### have been loaded against the Recording...
     if graph_uri is None: graph_uri = self.get_graph_and_recording_uri(uri)[0]
     graph = self.get_resource_as_graph(uri, BSML.Signal, graph_uri)
     if graph is None: return None
+
+    ## Have to get class of signal's recording...
     if signal_class is None: signal_class = Signal
+
     return signal_class.create_from_graph(uri, graph, units=None, **kwds)  # units set from graph...
 
   def signals(self, rec_uri, graph_uri=None):
@@ -161,15 +175,17 @@ class BSMLStore(GraphStore):
     # The following line works around a Virtuoso problem
     if graph_uri is None: graph_uri = self.get_graph_and_recording_uri(uri)[0]
     graph = self.get_resource_as_graph(uri, BSML.Event, graph_uri)
+
+##### Put into Event.load_from_graph()   ????
     for tm in graph.get_objects(uri, BSML.time):  ## This could be improved...
       graph.append_graph(self.get_resource_as_graph(tm.uri, BSML.Instant, graph_uri))
       graph.append_graph(self.get_resource_as_graph(tm.uri, BSML.Interval, graph_uri))
     return Event.create_from_graph(uri, graph)
 
-  def events(self, rec_uri, eventtype=None, timetype=None, graph_uri=None):
-  #------------------------------------------------------------------------
+  def get_event_uris(self, rec_uri, eventtype=None, timetype=None, graph_uri=None):
+  #--------------------------------------------------------------------------------
     '''
-    Return a list of all Events associated with a recording.
+    Return a list of Event URIs associated with a recording.
 
     :param uri: The URI of the recording.
     :param eventtype: The type of events to find. Optional.
@@ -189,6 +205,10 @@ class BSMLStore(GraphStore):
         prefixes=dict(bsml=BSML.prefix), graph=graph_uri)
       ]
 
+  ### Add a get_events() similar to get_annotations(), ideally be using
+  ### a generic method -- get_resources(cls, ...) ???
+
+
   def event_types(self, rec_uri, counts=False, graph_uri=None):
   #------------------------------------------------------------
     '''
@@ -201,10 +221,10 @@ class BSMLStore(GraphStore):
     '''
     return [ tuple(r[1:3]) if counts else r[1]
       for r in self.get_resources(BSML.Event, rvars='?et count(?et) as ?count',
-        condition = '?r bsml:recording <%s> . ?r bsml:eventType ?et' % rec_uri,
+        condition = '?e bsml:recording <%s> . ?e bsml:eventType ?et' % rec_uri,
         group = '?et',
         prefixes = dict(bsml=BSML.prefix),
-        graph = graph_uri
+        graph = graph_uri, resource='?e'
         ) ]
 
 
@@ -230,41 +250,27 @@ class BSMLStore(GraphStore):
 
     return Annotation.create_from_graph(uri, graph)
 
-#  def get_annotation_by_content(self, uri, graph_uri=None):
-#  #--------------------------------------------------------
-#    '''
-#    Get an Annotation from the repository identified by its body content.
-#
-#    :param uri: The URI of the body of an Annotation.
-#    :rtype: :class:`~biosignalml.Annotation`
-#    '''
-#    if graph_uri is None: graph_uri = self.get_graph_and_recording_uri(uri)[0]
-#    for r in self.select('?a', 'graph <%(g)s> { ?a a oa:Annotation . ?a oa:hasBody <%(u)s> }',
-#                         params = dict(g=graph_uri, u=uri),
-#                         prefixes = dict(oa = OA.prefix)):
-#      return self.get_annotation(sparqlstore.get_result_value(r, 'a'), graph_uri)
-
-  def annotations(self, uri, graph_uri=None):
-  #------------------------------------------
+  def get_annotation_uris(self, subject, graph_uri=None):
+  #------------------------------------------------------
     """
-    Return a list of all Annotations about a subject.
+    Return all Annotation URIs about a subject.
 
-    :param uri: The URI of the subject. Fragment parts of the subject URI
-      are ignored.
+    :param subject: The URI of the subject.
     :param graph_uri: An optional URI of the graph to query.
     :rtype: list of bsml:Annotation URIs
     """
     return [ r[1]
       for r in self.get_resources(BSML.Annotation, rvars='?ann',
-        condition='''?ann dct:subject <%(subj)s>
-                       minus { [] prv:precededBy ?ann }
+        condition='''?ann a bsml:Annotation ;
+                          dct:subject <%(subj)s>
+                            minus { [] prv:precededBy ?ann }
                    } union {
                      ?ann a bsml:Annotation ;
                           dct:subject [ a bsml:Segment ;
                                         dct:source <%(subj)s>
                                       ]
                             minus { [] prv:precededBy ?ann }'''
-                   % dict(subj=uri),
+                   % dict(subj=subject),
 ## Following produces an internal Virtuoso error...
 #                    {
 #                      { ?ann dct:subject <%(subj)s> }
@@ -276,6 +282,59 @@ class BSMLStore(GraphStore):
         graph = graph_uri
         ) ]
 
+  def get_annotations(self, subject, graph_uri=None):
+  #--------------------------------------------------
+    """
+    Return all Annotations about a subject.
+
+    This is significantly faster (approx 15x) then getting a list of URIs
+    (fast) and then creating Annotations (slow).
+
+    Code could be used as a template for general abstract object creation
+    based on PropertyMap and bypassing internal use of RDF -- instead map
+    directly to and from SPARQL.
+
+    :param subject: The URI of the subject.
+    :param graph_uri: An optional URI of the graph to query.
+    :rtype: list of bsml:Annotations
+    """
+    anns = self.get_resources(BSML.Annotation,
+        rvars='?ann ?about ?stype ?comment ?tag ?created ?source ?tm ?start ?duration ?timeline',
+        condition='''minus { [] prv:precededBy ?ann }
+                       {
+                         ?ann dct:subject ?about .
+                         ?about a ?stype .
+                         optional { ?ann rdfs:comment ?comment }
+                         optional { ?ann dct:created ?created }
+                         optional { ?ann bsml:tag ?tag }
+                         { ?ann dct:subject <%(subject)s> }
+                         union
+                         { ?about a bsml:Segment ;
+                             dct:source <%(subject)s> ;
+                             dct:source ?source ;
+                             bsml:time ?tm .
+                             ?tm tl:timeline ?timeline .
+                           { ?tm a bsml:Interval ; tl:start ?start ; tl:duration ?duration }
+                           union
+                           { ?tm a bsml:Instant ; tl:at ?start }
+                         }
+                       }''' % dict(subject=subject),
+        prefixes = dict(bsml=BSML.prefix, dct=DCT.prefix, prv=PRV.prefix, tl=TL.prefix),
+        graph = graph_uri,
+#       order = '?start',  ## Virtuoso doesn't correctly sort xsd:dayTimeDuration
+        )
+    anns.sort(key=operator.itemgetter(9, 2, 1))  # start, about, uri
+    annotations = [ ]
+    for a in anns:
+      ann = Annotation(a[1], about=a[2], comment=a[4], created=a[6])
+      if str(a[3]) == str(BSML.Segment):
+        time = TemporalEntity.create(a[8], float(a[9]),
+          duration=None if a[10] in ['', None] else float(a[10]),
+          timeline=a[11])
+        ann.about = Segment(a[2], source=a[7], time=time)
+      annotations.append(ann)
+    return annotations
+
   def get_semantic_tags(self):
   #---------------------------
     """
@@ -297,7 +356,7 @@ class BSMLUpdateStore(BSMLStore, GraphUpdate):
   def extend_recording_graph(self, recording, *abstractobjects):
   #-------------------------------------------------------------
     """
-    Add metadata about an object to a recording's graph.
+    Add abstractobjects to a recording's graph.
     """
     for abstractobject in abstractobjects:
       self._sparqlstore.extend_graph(recording.graph.uri,
@@ -316,15 +375,15 @@ class BSMLUpdateStore(BSMLStore, GraphUpdate):
       [("bsml:deletedResource", "prv:precededBy", "<%s>" % uri)],
       prefixes=dict(bsml=BSML.prefix, prv=PRV.prefix))
 
-  def add_recording_graph(self, uri, rdf, creator, format=Format.RDFXML):
-  #----------------------------------------------------------------------
+  def add_recording_graph(self, uri, graph_rdf, creator, format=Format.RDFXML):
+  #---------------------------------------------------------------------------
     """
     Add RDF metadata describing a recording as a new named graph.
 
     Provenance statements will be made about the new graph, including
     linking it to any previous graph describing the recording.
     """
-    return self.add_resource_graph(uri, BSML.Recording, rdf, creator, format=format)
+    return self.add_resource_graph(uri, BSML.Recording, graph_rdf, creator, format=format)
 
   def store_recording(self, recording, creator=None):
   #--------------------------------------------------
