@@ -18,16 +18,9 @@
 #
 ######################################################
 
-"""
-What this is about...
-
-"""
-
 import logging
 import json
 
-import RDF as librdf
-
 from biosignalml import BSML
 import biosignalml.model as model
 from biosignalml.model.mapping import PropertyMap
