@@ -48,15 +48,14 @@ expression notation, a block is defined as::
 
 Supported block types are defined in the :class:`BlockType` class.
 
-A block's header is a set of '(name, value)' pairs, which is sent formatted as JSON, and in
-Python kept as a ``{ name: value }`` dictionary. Valid 'names' and 'values'
-are specific to each block type.
+A block's header is a set of '(name, value)' pairs, which is sent formatted as JSON, and
+kept in Python as a ``{name: value}`` dictionary. Valid 'names' and 'values'
+are specific to each block type, and are described with the :class:`BlockType` class.
 
 """
 
 VERSION = 1    #: Initial version of Block Stream protocol.
 
-
 import logging
 import hashlib
 import Queue
@@ -68,6 +67,13 @@ except ImportError:
 import numpy as np
 
 
+__all__ = [ 'BlockParser', 'BlockStream', 'BlockType', 'Checksum', 'Error',
+            'ErrorBlock', 'SignalData', 'SignalDataBlock',
+            'SignalDataStream', 'StreamBlock', 'TestBlock', 'StreamException',
+            'VERSION' ]
+
+
+
 class BlockType(object):
 #=======================
   '''
@@ -142,10 +148,6 @@ class BlockType(object):
   contains discontinuous segments they will be returned in separate blocks.
   """
 
-  INFO = 'I'
-
-  RDF  = 'R'
-
   DATA = 'D'
   """
   Time series data.
@@ -161,11 +163,6 @@ class BlockType(object):
 
       REQUIRED.
 
-    **info** (*integer*)
-      The index into arrays describing the signal that were in the latest Info block.
-
-      REQUIRED.
-
     **start** (*float*)
       The time, in seconds, from the start of the signal's recording, of the first
       sample value.
@@ -332,7 +329,6 @@ class ErrorBlock(StreamBlock):
   """
   An error message block.
 
-  .. todo:: Document parameters.
   """
   def __init__(self, errblock, msg):
   #---------------------------------
@@ -341,30 +337,6 @@ class ErrorBlock(StreamBlock):
     StreamBlock.__init__(self, 0, BlockType.ERROR, header, bytearray(msg))
 
 
-class InfoBlock(StreamBlock):
-#============================
-  """
-  An info message block.
-
-  .. todo:: Document parameters.
-  """
-  def __init__(self, **header):
-  #----------------------------
-    StreamBlock.__init__(self, 0, BlockType.INFO, header, '')
-
-
-class RDFBlock(StreamBlock):
-#===========================
-  """
-  An RDF message block.
-
-  .. todo:: Document parameters.
-  """
-  def __init__(self, rdf, mimetype):
-  #---------------------------------
-    StreamBlock.__init__(self, 0, BlockType.RDF, {'mimetype': mimetype}, bytearray(rdf))
-
-
 class SignalDataBlock(StreamBlock):
 #==================================
   """
@@ -385,7 +357,6 @@ class SignalDataBlock(StreamBlock):
     start = self.header.get('start', 0)
     count = self.header.get('count', 0)
     dims = self.header.get('dims', 1)
-    info = self.header.get('info', None)
     dtype = self.header.get('dtype', None)
     rate = self.header.get('rate', None)
     ctype = self.header.get('ctype', None)
@@ -412,7 +383,7 @@ class SignalDataBlock(StreamBlock):
         data = np.frombuffer(self.content[datastart:], dtype=dt)
       else:
         data = np.reshape(np.frombuffer(self.content[datastart:], dtype=dt), (count, dims))
-    return SignalData(uri, start, data, info=info, rate=rate, clock=clock, dtype=dtype, ctype=ctype)
+    return SignalData(uri, start, data, rate=rate, clock=clock, dtype=dtype, ctype=ctype)
 
 
 class BlockParser(object):
@@ -632,8 +603,8 @@ class SignalData(object):
   :param dtype: The requested data type which to use for stream data values.
   :param ctype: The requested data type which to use for stream time values.
   '''
-  def __init__(self, uri, start, data, info=None, rate=None, clock=None, dtype=None, ctype=None):
-  #----------------------------------------------------------------------------------------------
+  def __init__(self, uri, start, data, rate=None, clock=None, dtype=None, ctype=None):
+  #-----------------------------------------------------------------------------------
     if rate is None and clock is None:
       raise StreamException('Data must have either a rate or a clock')
     elif rate is not None and clock is not None:
@@ -644,7 +615,6 @@ class SignalData(object):
       if clock.ndim != 1: raise StreamException('Clock must be a 1-D array')
       if len(clock) != len(data): raise StreamException('Clock and data have different lengths')
     self.uri = uri
-    self.info = info
     self.start = start
     self.data = data
     self.rate = rate
@@ -680,7 +650,6 @@ class SignalData(object):
                'count': len(self.data),
                'dtype': dtype.descr[0][1]
              }
-    if self.info is not None: header['info'] = self.info
     if self.data.ndim > 1: header['dims'] = self.data.shape[1]
     if self.rate: header['rate'] = self.rate
     if self.clock is not None:
@@ -799,7 +768,7 @@ if __name__ == '__main__':
 
   print nd
   testQ = Queue.Queue()
-  bp = BlockParser(testQ, Checksum.STRICT)
+  bp = BlockParser(testQ.put, Checksum.STRICT)
   bp.process(nd.streamblock().bytes(Checksum.STRICT))
 
   print testQ.get(True, 10).signaldata()
